import { Component, State, Prop, h, Listen, Watch, Element, Event, Fragment } from '@stencil/core';
import { store } from '@stencil/redux';
import { saveQuestionSet, updateErrors, pushAnswer, getPartnerEngineConfig, removeActiveAnswerId, updateActiveAnswerId, getAnswersState, getApplicationFormState, getQuestionPoolState, getQuestionSetState, doesUserOwnApplicationForm, userHasPermission, checkAnswerQueue, } from '@boldpenguin/sdk-redux';
import { LoadingStates, BpSdkPermissions, BpSdkPermissionAction, AnalyticsTrackingEventName, ApplicationFormProcessState, } from '@boldpenguin/sdk-types';
import { updateAnswerValue, getErrors, isValidAnswer, focusErrors, hasAnswerChanged, getQuestionGroupsWithParentQuestionIdAsKey, getQuestionGroupWithParentQuestionAnswerIdAsKey, hasAnswer, } from '@boldpenguin/sdk-utility';
import { canGetStore } from '../../utils/store';
export class BpSdkQuestionSetContainer {
  constructor() {
    /**
     * Question sets injected from redux
     */
    this.questionSets = {
      list: [],
      active: '',
      activeAnswerId: '',
      completedQuestionSetAnswerHasChanged: false,
      parentQuestionSets: [],
    };
    this.areResourcesReady = true;
    this.useExternal = false;
    this.canSave = true;
    this.isLocked = false;
    this.isReady = true;
    this.isFirstRun = true;
    this.isLoadingApplication = false;
    this.heartbeatInterval = undefined;
    this.isUpdating = false;
    this.handleSubscriptions = () => {
      if (canGetStore()) {
        this.storeUnsubscribe && this.storeUnsubscribe();
        store.mapDispatchToProps(this, {
          saveQuestionSet,
          updateErrors,
          pushAnswer,
          removeActiveAnswerId,
          updateActiveAnswerId,
          checkAnswerQueue,
        });
        this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
      }
    };
    this.mapStateToProps = (state) => {
      return {
        questionSets: state.application.questionSets,
        config: getPartnerEngineConfig(state),
        enableUserApplicationGuard: !!state.config.options.enableUserApplicationGuard,
        locked: state.application.form.locked,
        isOwner: doesUserOwnApplicationForm(state),
        hasPermissionToEdit: userHasPermission(state, {
          permission: BpSdkPermissions.ApplicationForms,
          action: BpSdkPermissionAction.Update,
        }),
        pendingProcesses: state.application.pendingProcesses,
        queue: state.queues.answers.queue,
        answersResourceState: getAnswersState(state),
        applicationFormResourceState: getApplicationFormState(state),
        questionPoolResourceState: getQuestionPoolState(state),
        questionSetResourceState: getQuestionSetState(state),
      };
    };
    this.handleSaveQuestionSet = (questionSet) => {
      const invalidAnswers = questionSet.answers.filter((answer) => !isValidAnswer(answer));
      if (!invalidAnswers.length) {
        this.removeActiveAnswerId();
        this.saveQuestionSet({
          config: this.config,
          appFormQuestionSet: questionSet,
          navigationDirection: 'forward',
        });
        this.bpAnalyticsTrack.emit({
          eventName: AnalyticsTrackingEventName.QuestionSetSaved,
          eventPayload: {
            applicationFormQuestionSetId: questionSet.id,
            questionSetId: questionSet.question_set_id,
            questionSetCode: questionSet.question_set.code,
          },
        });
      }
      else {
        invalidAnswers.forEach((answer) => {
          this.updateErrors({ answer, errors: getErrors(answer) });
        });
        focusErrors();
      }
    };
  }
  resetState() {
    this.isReady = true;
    this.isFirstRun = true;
    this.isLoadingApplication = false;
  }
  connectedCallback() {
    this.resetState();
    this.handleSubscriptions();
  }
  componentWillLoad() {
    this.useExternal = !!customElements.get('bp-sdk-external-question-set-container');
    customElements.whenDefined('bp-sdk-external-question-set-container').then(() => {
      this.useExternal = true;
    });
    this.handleSubscriptions();
    this.onAnswersResourcesStateChange(this.answersResourceState);
    this.onApplicationFormResourceStateChange(this.applicationFormResourceState);
    this.onHasPermissionToEdit(this.hasPermissionToEdit);
    this.onEnableUserApplicationGuard(this.enableUserApplicationGuard);
    this.onIsFirstRun(this.isFirstRun);
    this.onIsOwner(this.isOwner);
    this.onIsReady(this.isReady);
    this.onLocked(this.locked);
    this.onQuestionPoolResourcesStateChange(this.questionPoolResourceState);
    this.onQuestionSetResourcesStateChange(this.questionSetResourceState);
    this.onQueueChange(this.queue);
  }
  onQueueChange(queue, oldQueue) {
    this.canSave = !(queue === null || queue === void 0 ? void 0 : queue.length) && !this.doesCurrentQuestionSetHaveAPendingProcess();
    // While there is a queue, send a heartbeat to make sure redux is parsing it
    if ((queue === null || queue === void 0 ? void 0 : queue.length) && !(oldQueue === null || oldQueue === void 0 ? void 0 : oldQueue.length) && !this.heartbeatInterval) {
      this.heartbeatInterval = setInterval(() => {
        this.checkAnswerQueue();
      }, 250);
    }
    // If the queue has emptied, stop the heartbeat
    if (!(queue === null || queue === void 0 ? void 0 : queue.length) && (oldQueue === null || oldQueue === void 0 ? void 0 : oldQueue.length) && this.heartbeatInterval) {
      clearInterval(this.heartbeatInterval);
      this.heartbeatInterval = undefined;
    }
  }
  onEnableUserApplicationGuard(enableUserApplicationGuard) {
    this.updateIsLocked(this.locked, this.isOwner, this.hasPermissionToEdit, enableUserApplicationGuard);
  }
  onHasPermissionToEdit(hasPermission) {
    this.updateIsLocked(this.locked, this.isOwner, hasPermission, this.enableUserApplicationGuard);
  }
  onIsOwner(isOwner) {
    this.updateIsLocked(this.locked, isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
  }
  onLocked(locked) {
    this.updateIsLocked(locked, this.isOwner, this.hasPermissionToEdit, this.enableUserApplicationGuard);
  }
  onPendingProcessesChange(processes) {
    this.canSave = !this.doesCurrentQuestionSetHaveAPendingProcess(processes);
  }
  updateIsLocked(locked, isOwner, hasPermission, enableUserApplicationGuard) {
    this.isLocked = locked || (enableUserApplicationGuard && (!isOwner || !hasPermission));
  }
  onIsFirstRun(isFirstRun) {
    this.updateAreResourcesReady(isFirstRun, this.isReady);
  }
  onIsReady(isReady) {
    this.updateAreResourcesReady(this.isFirstRun, isReady);
  }
  updateAreResourcesReady(isFirstRun, isReady) {
    this.areResourcesReady = isFirstRun || isReady;
  }
  disconnectedCallback() {
    if (this.heartbeatInterval)
      clearInterval(this.heartbeatInterval);
    this.storeUnsubscribe && this.storeUnsubscribe();
  }
  onRemoveActiveAnswerRequest() {
    this.removeActiveAnswerId();
  }
  onUpdateActiveAnswerRequest({ detail }) {
    const answer = this.questionSets.list.reduce((acc, list) => acc || list.answers.find((answer) => answer.id === detail), null);
    if (answer) {
      this.updateActiveAnswerId(answer.id);
    }
  }
  onAnswersResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: state,
      applicationForm: this.applicationFormResourceState,
      questionPools: this.questionPoolResourceState,
      questionSets: this.questionSetResourceState,
    });
  }
  onApplicationFormResourceStateChange(state) {
    if (this.isFirstRun && state === LoadingStates.Loaded) {
      this.isFirstRun = false;
    }
    if (state === LoadingStates.Loading) {
      this.isLoadingApplication = true;
    }
    else {
      this.isLoadingApplication = false;
    }
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: state,
      questionPools: this.questionPoolResourceState,
      questionSets: this.questionSetResourceState,
    });
  }
  onQuestionPoolResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: this.applicationFormResourceState,
      questionPools: state,
      questionSets: this.questionSetResourceState,
    });
  }
  onQuestionSetResourcesStateChange(state) {
    this.isReady = this.calculateResourcesReady({
      answers: this.answersResourceState,
      applicationForm: this.applicationFormResourceState,
      questionPools: this.questionPoolResourceState,
      questionSets: state,
    });
  }
  calculateResourcesReady(resources = {}) {
    if (!resources ||
      resources.applicationForm !== LoadingStates.Loaded ||
      resources.answers === LoadingStates.Loading ||
      resources.questionPools === LoadingStates.Loading ||
      resources.questionSets === LoadingStates.Loading) {
      return false;
    }
    return true;
  }
  handleSaveAnswer(event) {
    if (this.useExternal) {
      return;
    }
    const { answer, new_value } = event.detail;
    const updatedAnswer = updateAnswerValue(answer, new_value);
    if (hasAnswerChanged(answer, new_value) || (hasAnswer(updatedAnswer) && !answer.is_completed)) {
      this.pushAnswer(updatedAnswer);
    }
  }
  componentWillUpdate() {
    this.handleSubscriptions();
  }
  render() {
    return [this.useExternal && this.renderExternal(), !this.useExternal && this.renderInternal()];
  }
  renderExternal() {
    return (h("bp-sdk-external-question-set-container", { answersResourceState: this.answersResourceState, applicationFormResourceState: this.applicationFormResourceState, config: this.config, enableUserApplicationGuard: this.enableUserApplicationGuard, hasPermissionToEdit: this.hasPermissionToEdit, isOwner: this.isOwner, locked: this.locked, pendingProcesses: this.pendingProcesses, questionPoolResourceState: this.questionPoolResourceState, questionSetResourceState: this.questionSetResourceState, questionSets: this.questionSets, queue: this.queue }));
  }
  renderQuestionSetHeader(set) {
    var _a, _b, _c, _d;
    return (h("bp-sdk-question-set-header", { applicationId: set.application_form_id, code: (_a = set.question_set) === null || _a === void 0 ? void 0 : _a.code, isActive: set.is_active, isCompleted: set.is_completed, name: (_b = set.question_set) === null || _b === void 0 ? void 0 : _b.name, setId: set.question_set_id, setTitle: (_c = set.question_set) === null || _c === void 0 ? void 0 : _c.title, tenantId: (_d = set.question_set) === null || _d === void 0 ? void 0 : _d.tenant_id }));
  }
  shouldRenderQuestionGroup(questionGroupsWithAnswersAsKey, answer) {
    return questionGroupsWithAnswersAsKey.has(answer.id);
  }
  renderQuestionSet(set) {
    var _a;
    // Gets multiple separate groups/nested groups
    const questionGroupsWithAnswersAsKey = getQuestionGroupWithParentQuestionAnswerIdAsKey(set, getQuestionGroupsWithParentQuestionIdAsKey(set));
    switch ((_a = set.question_set) === null || _a === void 0 ? void 0 : _a.code) {
      // These two special cases are the legacy question groups CA components,
      // the question pools CA flow goes down into the question-set component
      case 'driver_master_question_set':
        return (h("bp-sdk-driver-question-set-container", { areResourcesReady: this.areResourcesReady, canSave: this.canSave, locked: this.isLocked, questionSet: set, saveHandler: this.handleSaveQuestionSet, questionGroupsWithAnswersAsKey: questionGroupsWithAnswersAsKey }));
      case 'vehicle_master_question_set':
        return (h("bp-sdk-vehicle-question-set", { areResourcesReady: this.areResourcesReady, canSave: this.canSave, locked: this.isLocked, questionSet: set, saveHandler: this.handleSaveQuestionSet, questionGroupsWithAnswersAsKey: questionGroupsWithAnswersAsKey }));
      default:
        return (h("bp-sdk-question-set", { areResourcesReady: this.areResourcesReady, canSave: this.canSave, locked: this.isLocked, questionSet: set, saveHandler: this.handleSaveQuestionSet, questionGroupsWithAnswersAsKey: questionGroupsWithAnswersAsKey, shouldRenderQuestionGroup: this.shouldRenderQuestionGroup }));
    }
  }
  renderPendingProcessesContainer() {
    return h("bp-sdk-pending-process-container", { pendingProcesses: this.pendingProcesses });
  }
  renderInternal() {
    if (!canGetStore()) {
      return null;
    }
    let activeQuestionSet = this.questionSets.list[this.questionSets.list.length - 1];
    if (this.isFirstRun && this.isLoadingApplication) {
      return h("bp-sdk-application-load", null);
    }
    if (this.questionSets.active && this.questionSets.active !== '') {
      activeQuestionSet = this.questionSets.list.find((qs) => qs.question_set_id === this.questionSets.active);
    }
    if (activeQuestionSet) {
      return (h(Fragment, null,
        this.renderQuestionSetHeader(activeQuestionSet),
        this.renderQuestionSet(activeQuestionSet),
        this.renderPendingProcessesContainer()));
    }
  }
  doesCurrentQuestionSetHaveAPendingProcess(processes) {
    const currentProcesses = processes ? processes : this.pendingProcesses;
    if (!(currentProcesses === null || currentProcesses === void 0 ? void 0 : currentProcesses.length)) {
      return false;
    }
    return currentProcesses.some((process) => process.state !== ApplicationFormProcessState.Completed && process.question_set_id === this.questionSets.active);
  }
  static get is() { return "bp-sdk-question-set-container"; }
  static get properties() { return {
    "questionSets": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IQuestionSetState",
        "resolved": "IQuestionSetState",
        "references": {
          "IQuestionSetState": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question sets injected from redux"
      },
      "defaultValue": "{\n    list: [],\n    active: '',\n    activeAnswerId: '',\n    completedQuestionSetAnswerHasChanged: false,\n    parentQuestionSets: [],\n  }"
    },
    "config": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IPartnerEngineConfig",
        "resolved": "IPartnerEngineConfig",
        "references": {
          "IPartnerEngineConfig": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "IPartnerEngineConfig injected from redux"
      }
    },
    "answersResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Answer resource states injected from redux"
      },
      "attribute": "answers-resource-state",
      "reflect": false
    },
    "applicationFormResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Application form resource states injected from redux"
      },
      "attribute": "application-form-resource-state",
      "reflect": false
    },
    "questionPoolResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question pool resource states injected from redux"
      },
      "attribute": "question-pool-resource-state",
      "reflect": false
    },
    "questionSetResourceState": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "LoadingStates",
        "resolved": "LoadingStates.Error | LoadingStates.Loaded | LoadingStates.Loading | LoadingStates.NotLoaded",
        "references": {
          "LoadingStates": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Question set resource states injected from redux"
      },
      "attribute": "question-set-resource-state",
      "reflect": false
    },
    "locked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Application form state injected from redux"
      },
      "attribute": "locked",
      "reflect": false
    },
    "pendingProcesses": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IApplicationFormProcesses[]",
        "resolved": "IApplicationFormProcesses[]",
        "references": {
          "IApplicationFormProcesses": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Pending processes of application form state injected from redux"
      }
    },
    "queue": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "IAnswer[]",
        "resolved": "IAnswer[]",
        "references": {
          "IAnswer": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Answer queue"
      }
    },
    "isOwner": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Is owner of application"
      },
      "attribute": "is-owner",
      "reflect": false
    },
    "hasPermissionToEdit": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Has permission to edit application"
      },
      "attribute": "has-permission-to-edit",
      "reflect": false
    },
    "enableUserApplicationGuard": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enabled user check for application"
      },
      "attribute": "enable-user-application-guard",
      "reflect": false
    }
  }; }
  static get states() { return {
    "areResourcesReady": {},
    "useExternal": {},
    "canSave": {},
    "isLocked": {},
    "isReady": {},
    "isFirstRun": {},
    "isLoadingApplication": {},
    "heartbeatInterval": {},
    "isUpdating": {}
  }; }
  static get events() { return [{
      "method": "bpAnalyticsTrack",
      "name": "bpAnalyticsTrack",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits an analytics tracking event, see `AnalyticsTrackingEventName` (sdk-types) for event names"
      },
      "complexType": {
        "original": "IAnalyticsTrackEventDetail",
        "resolved": "IAnalyticsTrackEventDetail",
        "references": {
          "IAnalyticsTrackEventDetail": {
            "location": "import",
            "path": "@boldpenguin/sdk-types"
          }
        }
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "queue",
      "methodName": "onQueueChange"
    }, {
      "propName": "enableUserApplicationGuard",
      "methodName": "onEnableUserApplicationGuard"
    }, {
      "propName": "hasPermissionToEdit",
      "methodName": "onHasPermissionToEdit"
    }, {
      "propName": "isOwner",
      "methodName": "onIsOwner"
    }, {
      "propName": "locked",
      "methodName": "onLocked"
    }, {
      "propName": "pendingProcesses",
      "methodName": "onPendingProcessesChange"
    }, {
      "propName": "isFirstRun",
      "methodName": "onIsFirstRun"
    }, {
      "propName": "isReady",
      "methodName": "onIsReady"
    }, {
      "propName": "answersResourceState",
      "methodName": "onAnswersResourcesStateChange"
    }, {
      "propName": "applicationFormResourceState",
      "methodName": "onApplicationFormResourceStateChange"
    }, {
      "propName": "questionPoolResourceState",
      "methodName": "onQuestionPoolResourcesStateChange"
    }, {
      "propName": "questionSetResourceState",
      "methodName": "onQuestionSetResourcesStateChange"
    }]; }
  static get listeners() { return [{
      "name": "removeActiveAnswer",
      "method": "onRemoveActiveAnswerRequest",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "updateActiveAnswer",
      "method": "onUpdateActiveAnswerRequest",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "saveAnswer",
      "method": "handleSaveAnswer",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
